@@ -9,13 +9,12 @@
 
 __all__ = [
     'CompileOnlyRegressionTest', 'RegressionTest', 'RunOnlyRegressionTest',
-    'DEPEND_BY_ENV', 'DEPEND_EXACT', 'DEPEND_FULLY', 'final',
     'RegressionMixin'
 ]
 
 
-import functools
 import glob
+import hashlib
 import inspect
 import itertools
 import numbers
@@ -44,8 +43,6 @@ from reframe.core.exceptions import (BuildError, DependencyError,
                                      ReframeSyntaxError)
 from reframe.core.meta import RegressionTestMeta
 from reframe.core.schedulers import Job
-from reframe.core.variables import DEPRECATE_WR
-from reframe.core.warnings import user_deprecation_warning
 
 
 class _NoRuntime(ContainerPlatform):
@@ -62,38 +59,8 @@ class _NoRuntime(ContainerPlatform):
         raise NotImplementedError
 
 
-# Dependency kinds
-
-#: Constant to be passed as the ``how`` argument of the
-#: :func:`~RegressionTest.depends_on` method. It denotes that test case
-#: dependencies will be explicitly specified by the user.
-#:
-#:  This constant is directly available under the :mod:`reframe` module.
-#:
-#: .. deprecated:: 3.3
-#:    Please use a callable as the ``how`` argument.
-DEPEND_EXACT = 1
-
-#: Constant to be passed as the ``how`` argument of the
-#: :func:`RegressionTest.depends_on` method. It denotes that the test cases of
-#: the current test will depend only on the corresponding test cases of the
-#: target test that use the same programming environment.
-#:
-#:  This constant is directly available under the :mod:`reframe` module.
-#:
-#: .. deprecated:: 3.3
-#:    Please use a callable as the ``how`` argument.
-DEPEND_BY_ENV = 2
-
-#: Constant to be passed as the ``how`` argument of the
-#: :func:`RegressionTest.depends_on` method. It denotes that each test case of
-#: this test depends on all the test cases of the target test.
-#:
-#:  This constant is directly available under the :mod:`reframe` module.
-#:
-#: .. deprecated:: 3.3
-#:    Please use a callable as the ``how`` argument.
-DEPEND_FULLY = 3
+# Meta-type to use for type checking in some variables
+Deferrable = typ.make_meta_type('Deferrable', _DeferredExpression)
 
 
 # Valid systems/environments mini-language
@@ -125,21 +92,6 @@ _USER_PIPELINE_STAGES = (
 )
 
 
-def final(fn):
-    fn._rfm_final = True
-    user_deprecation_warning(
-        'using the @rfm.final decorator from the rfm module is '
-        'deprecated; please use the built-in decorator @final instead.',
-        from_version='3.7.0'
-    )
-
-    @functools.wraps(fn)
-    def _wrapped(*args, **kwargs):
-        return fn(*args, **kwargs)
-
-    return _wrapped
-
-
 _RFM_TEST_KIND_MIXIN = 0
 _RFM_TEST_KIND_COMPILE = 1
 _RFM_TEST_KIND_RUN = 2
@@ -236,28 +188,7 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
 
         return ret
 
-    #: The name of the test.
-    #:
-    #: This is an alias of :attr:`unique_name`.
-    #:
-    #: .. warning::
-    #:
-    #:   Setting the name of a test is deprecated and will be disabled in the
-    #:   future. If you were setting the name of a test to circumvent the old
-    #:   long parameterized test names in order to reference them in
-    #:   dependency chains, please refer to :ref:`param_deps` for more details
-    #:   on how to achieve this.
-    #:
-    #: .. versionchanged:: 3.10.0
-    #:    Setting the :attr:`name` attribute is deprecated.
-    #:
-    name = deprecate(variable(typ.Str[r'[^\/]+'],
-                              attr_name='_rfm_unique_name', loggable=True),
-                     "setting the 'name' attribute is deprecated and "
-                     "will be disabled in the future", DEPRECATE_WR)
-
-    #: List of environments or environment features or environment properties
-    #: required by this test.
+    #: List of programming environments supported by this test.
     #:
     #: The syntax of this attribute is exactly the same as of the
     #: :attr:`valid_systems` except that the ``a:b`` entries are invalid.
@@ -355,8 +286,11 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
     #: A detailed description of the test.
     #:
     #: :type: :class:`str`
-    #: :default: ``self.display_name``
-    descr = variable(str, loggable=True)
+    #: :default: ``''``
+    #:
+    #: .. versionchanged:: 4.0
+    #:    The default value is now the empty string.
+    descr = variable(str, value='', loggable=True)
 
     #: The path to the source file or source directory of the test.
     #:
@@ -626,9 +560,12 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
     #: For more information on test resources, have a look at the
     #: :attr:`extra_resources` attribute.
     #:
-    #: :type: integral
-    #: :default: ``0``
-    num_gpus_per_node = variable(int, value=0, loggable=True)
+    #: :type: integral or :const:`None`
+    #: :default: :const:`None`
+    #:
+    #: .. versionchanged:: 4.0.0
+    #:    The default value changed to :const:`None`.
+    num_gpus_per_node = variable(int, type(None), value=None, loggable=True)
 
     #: Number of CPUs per task required by this test.
     #:
@@ -729,19 +666,39 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
     #: obtained performance value in order to actually assess whether the test
     #: passes the performance check or not.
     #:
-    #: :type: A scoped dictionary with system names as scopes or :class:`None`
-    #: :default: ``{}``
+    #: :type: A scoped dictionary with system names as scopes, performance
+    #:   variables as keys and reference tuples as values.
+    #:   The elements of reference tuples cannot be deferrable expressions.
     #:
     #: .. note::
     #:     .. versionchanged:: 3.0
     #:        The measurement unit is required. The user should explicitly
     #:        specify :class:`None` if no unit is available.
-    reference = variable(typ.Tuple[object, object, object],
-                         typ.Dict[str, typ.Dict[
-                             str, typ.Tuple[object, object, object, object]]
-    ], field=fields.ScopedDictField, value={})
-    # FIXME: There is not way currently to express tuples of `float`s or
-    # `None`s, so we just use the very generic `object`
+    #:
+    #:     .. versionchanged:: 3.8.0
+    #:        The unit in the reference tuple is again optional, but it is
+    #:        recommended to use it for clarity.
+    #:
+    #:     .. versionchanged:: 4.0.0
+    #:        Deferrable expressions are not allowed in reference tuples.
+    reference = variable(
+        typ.Tuple[~Deferrable, ~Deferrable, ~Deferrable],
+        typ.Dict[str, typ.Dict[str, typ.Tuple[~Deferrable, ~Deferrable,
+                                              ~Deferrable, ~Deferrable]]],
+        field=fields.ScopedDictField, value={}
+    )
+
+    #: Require that a reference is defined for each system that this test is
+    #: run on.
+    #:
+    #: If this is set and a reference is not found for the current system, the
+    #: test will fail.
+    #:
+    #: :type: boolean
+    #: :default: :const:`False`
+    #:
+    #: .. versionadded:: 4.0.0
+    require_reference = variable(typ.Bool, value=False)
 
     #:
     #: Refer to the :doc:`ReFrame Tutorials </tutorials>` for concrete usage
@@ -772,9 +729,6 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
 
     #: Patterns for verifying the performance of this test.
     #:
-    #: Refer to the :doc:`ReFrame Tutorials </tutorials>` for concrete usage
-    #: examples.
-    #:
     #: If set to :class:`None`, no performance checking will be performed.
     #:
     #: :type: A dictionary with keys of type :class:`str` and deferrable
@@ -782,6 +736,14 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
     #:     </deferrable_functions_reference>`) as values.
     #:     :class:`None` is also allowed.
     #: :default: :class:`None`
+    #:
+    #: .. warning::
+    #:
+    #:    You are advised to follow the new syntax for defining performance
+    #:    variables in your tests using either the :func:`@performance_function
+    #:    <reframe.core.builtins.performance_function>` builtin or the
+    #:    :attr:`perf_variables`, as :attr:`perf_patterns` will likely be
+    #:    deprecated in the future.
     perf_patterns = variable(typ.Dict[str, _DeferredExpression], type(None))
 
     #: The performance variables associated with the test.
@@ -795,7 +757,7 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
     #: By default, ReFrame will populate this field during the test's
     #: instantiation with all the member functions decorated with the
     #: :func:`@performance_function
-    #: <reframe.core.pipeline.RegressionMixin.performance_function>` decorator.
+    #: <reframe.core.builtins.performance_function>` decorator.
     #: If no performance functions are present in the class, no performance
     #: checking or reporting will be carried out.
     #:
@@ -813,7 +775,7 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
     #:     :ref:`deferrable-performance-functions`).
     #: :default: Collection of performance variables associated to each of
     #:     the member functions decorated with the :func:`@performance_function
-    #:     <reframe.core.pipeline.RegressionMixin.performance_function>`
+    #:     <reframe.core.builtins.performance_function>`
     #:     decorator.
     #:
     #: .. versionadded:: 3.8.0
@@ -824,25 +786,43 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
     #:
     #: These modules will be loaded during the :func:`setup` phase.
     #:
-    #: :type: :class:`List[str]`
+    #: :type: :class:`List[str]` or :class:`Dict[str, object]`
     #: :default: ``[]``
     modules = variable(typ.List[str], typ.List[typ.Dict[str, object]],
                        value=[], loggable=True)
 
     #: Environment variables to be set before running this test.
     #:
-    #: These variables will be set during the :func:`setup` phase.
+    #: The value of the environment variables can be of any type. ReFrame will
+    #: invoke :func:`str` on it whenever it needs to emit it in a script.
     #:
-    #: :type: :class:`Dict[str, str]`
+    #: :type: :class:`Dict[str, object]`
     #: :default: ``{}``
-    variables = variable(typ.Dict[str, str], value={}, loggable=True)
+    #:
+    #: .. versionadded:: 4.0.0
+    env_vars = variable(typ.Dict[str, str],
+                        typ.Dict[str, object], value={}, loggable=True)
+    # NOTE: We still keep the original type, just to allow setting this
+    # variable from the command line, because otherwise, ReFrame will not know
+    # how to convert a value to an arbitrary object.
+
+    #: Environment variables to be set before running this test.
+    #:
+    #: This is an alias of :attr:`env_vars`.
+    #:
+    #: .. deprecated:: 4.0.0
+    #:    Please use :attr:`env_vars` instead.
+    variables = deprecate(variable(alias=env_vars, loggable=True),
+                          f"the use of 'variables' is deprecated; "
+                          f"please use 'env_vars' instead")
 
     #: Time limit for this test.
     #:
     #: Time limit is specified as a string in the form
-    #: ``<days>d<hours>h<minutes>m<seconds>s`` or as number of seconds.
-    #: If set to :class:`None`, the |time_limit|_
-    #: of the current system partition will be used.
+    #: ``<days>d<hours>h<minutes>m<seconds>s`` or as number of seconds. If set
+    #: to :class:`None`, the
+    #: :attr:`~reframe.core.systems.SystemPartition.time_limit` of the current
+    #: system partition will be used.
     #:
     #: :type: :class:`str` or :class:`float` or :class:`int`
     #: :default: :class:`None`
@@ -863,9 +843,6 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
     #:    .. versionchanged:: 3.5.1
     #:       The default value is now :class:`None` and it can be set globally
     #:       per partition via the configuration.
-    #:
-    #:    .. |time_limit| replace:: :attr:`time_limit`
-    #:    .. _time_limit: #.systems[].partitions[].time_limit
     time_limit = variable(type(None), field=fields.TimerField,
                           value=None, loggable=True)
 
@@ -965,7 +942,8 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
     #: responsibility to check whether the build phase failed by adding an
     #: appropriate sanity check.
     #:
-    #: :type: boolean : :default: :class:`True`
+    #: :type: boolean
+    #: :default: :class:`True`
     build_locally = variable(typ.Bool, value=True, loggable=True)
 
     def __new__(cls, *args, **kwargs):
@@ -988,8 +966,7 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
         # Prepare initialization of test defaults (variables and parameters are
         # injected after __new__ has returned, so we schedule this function
         # call as a pre-init hook).
-        obj.__deferred_rfm_init = obj.__rfm_init__(*args,
-                                                   prefix=prefix, **kwargs)
+        obj.__deferred_rfm_init = obj.__rfm_init__(prefix)
 
         # Build pipeline hook registry and add the pre-init hook
         cls._rfm_pipeline_hooks = cls._process_hook_registry()
@@ -1033,21 +1010,7 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
             )
 
     @deferrable
-    def __rfm_init__(self, *args, prefix=None, **kwargs):
-        if not self.is_fixture() and not hasattr(self, '_rfm_unique_name'):
-            self._rfm_unique_name = type(self).variant_name(self.variant_num)
-
-            # Add the parameters from the parameterized_test decorator.
-            if args or kwargs:
-                arg_names = map(lambda x: util.toalphanum(str(x)),
-                                itertools.chain(args, kwargs.values()))
-                self._rfm_unique_name += '_' + '_'.join(arg_names)
-                self._rfm_old_style_params = True
-
-        # Pass if descr is a required variable.
-        if not hasattr(self, 'descr'):
-            self.descr = self.display_name
-
+    def __rfm_init__(self, prefix=None):
         self._perfvalues = {}
 
         # Static directories of the regression check
@@ -1073,9 +1036,6 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
         self._build_job = None
         self._compile_proc = None
 
-        # Performance logging
-        self._perf_logger = logging.null_logger
-
         # List of dependencies specified by the user
         self._userdeps = []
 
@@ -1085,7 +1045,7 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
         if rt.runtime().get_option('general/0/non_default_craype'):
             self._cdt_environ = Environment(
                 name='__rfm_cdt_environ',
-                variables={
+                env_vars={
                     'LD_LIBRARY_PATH': '$CRAY_LD_LIBRARY_PATH:$LD_LIBRARY_PATH'
                 }
             )
@@ -1166,6 +1126,15 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
         '''
         return self._rfm_unique_name
 
+    @loggable
+    @property
+    def name(self):
+        '''The name of the test.
+
+        This is an alias of :attr:`display_name`.
+        '''
+        return self.display_name
+
     @loggable
     @property
     def display_name(self):
@@ -1189,11 +1158,10 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
                 name += f'{prefix}{p}={format_fn(v)}'
 
             for f, v in info['fixtures'].items():
-                if isinstance(v, tuple):
-                    # This is join fixture
+                fixt = cls.fixture_space[f]
+                if fixt.action == 'join':
                     continue
 
-                fixt = cls.fixture_space[f]
                 name += _format_params(fixt.cls, v, f'{prefix}{f}.')
 
                 # Append any variables set for the fixtures
@@ -1202,9 +1170,6 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
 
             return name
 
-        if hasattr(self, '_rfm_old_style_params'):
-            return self.unique_name
-
         if hasattr(self, '_rfm_display_name'):
             return self._rfm_display_name
 
@@ -1222,6 +1187,42 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
 
         return self._rfm_display_name
 
+    @loggable
+    @property
+    def hashcode(self):
+        if hasattr(self, '_rfm_hashcode'):
+            return self._rfm_hashcode
+
+        m = hashlib.sha256()
+        if self.is_fixture():
+            m.update(self.unique_name.encode('utf-8'))
+        else:
+            basename, *params = self.display_name.split(' %')
+            m.update(basename.encode('utf-8'))
+            for p in sorted(params):
+                m.update(p.encode('utf-8'))
+
+        self._rfm_hashcode = m.hexdigest()[:8]
+        return self._rfm_hashcode
+
+    @loggable
+    @property
+    def short_name(self):
+        '''A short version of the test's display name.
+
+        The shortened version coincides with the :attr:`unique_name` for
+        simple tests and combines the test's class name and a hash code for
+        parameterised tests.
+
+        .. versionadded:: 4.0.0
+
+        '''
+
+        if self.unique_name != self.display_name:
+            return f'{type(self).__name__}_{self.hashcode}'
+        else:
+            return self.unique_name
+
     @property
     def current_environ(self):
         '''The programming environment that the regression test is currently
@@ -1473,7 +1474,7 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
            method may be called at any point of the test's lifetime.
         '''
 
-        ret = self.display_name
+        ret = f'{self.display_name} /{self.hashcode}'
         if self.current_partition:
             ret += f' @{self.current_partition.fullname}'
 
@@ -1584,11 +1585,11 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
             runtime = rt.runtime()
             self._stagedir = runtime.make_stagedir(
                 self.current_system.name, self._current_partition.name,
-                self._current_environ.name, self.unique_name
+                self._current_environ.name, self.short_name
             )
             self._outputdir = runtime.make_outputdir(
                 self.current_system.name, self._current_partition.name,
-                self._current_environ.name, self.unique_name
+                self._current_environ.name, self.short_name
             )
         except OSError as e:
             raise PipelineError('failed to set up paths') from e
@@ -1616,16 +1617,12 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
                           **job_opts)
 
     def _setup_build_job(self, **job_opts):
-        self._build_job = self._create_job(f'rfm_{self.unique_name}_build',
+        self._build_job = self._create_job(f'rfm_build',
                                            self.local or self.build_locally,
                                            **job_opts)
 
     def _setup_run_job(self, **job_opts):
-        self._job = self._create_job(f'rfm_{self.unique_name}_job',
-                                     self.local, **job_opts)
-
-    def _setup_perf_logging(self):
-        self._perf_logger = logging.getperflogger(self)
+        self._job = self._create_job(f'rfm_job', self.local, **job_opts)
 
     def _setup_container_platform(self):
         try:
@@ -1771,7 +1768,7 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
             self.build_system.executable = self.executable
 
         user_environ = Environment(self.unique_name,
-                                   self.modules, self.variables.items())
+                                   self.modules, self.env_vars.items())
         environs = [self._current_partition.local_env, self._current_environ,
                     user_environ, self._cdt_environ]
         self._build_job.time_limit = (
@@ -1779,6 +1776,11 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
                 f'systems/0/partitions/@{self.current_partition.name}'
                 f'/time_limit')
         )
+        # Get job options from managed resources and prepend them to
+        # build_job_opts. We want any user supplied options to be able to
+        # override those set by the framework.
+        resources_opts = self._map_resources_to_jobopts()
+        self._build_job.options = resources_opts + self._build_job.options
         with osext.change_dir(self._stagedir):
             # Prepare build job
             build_commands = [
@@ -1900,7 +1902,7 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
             *self.postrun_cmds
         ]
         user_environ = Environment(self.unique_name,
-                                   self.modules, self.variables.items())
+                                   self.modules, self.env_vars.items())
         environs = [
             self._current_partition.local_env,
             self._current_environ,
@@ -1913,7 +1915,7 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
                 environs.insert(2, cp_env)
 
         # num_gpus_per_node is a managed resource
-        if self.num_gpus_per_node > 0:
+        if self.num_gpus_per_node:
             self.extra_resources.setdefault(
                 '_rfm_gpu', {'num_gpus_per_node': self.num_gpus_per_node}
             )
@@ -1921,11 +1923,7 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
         # Get job options from managed resources and prepend them to
         # job_opts. We want any user supplied options to be able to
         # override those set by the framework.
-        resources_opts = []
-        for r, v in self.extra_resources.items():
-            resources_opts.extend(
-                self._current_partition.get_resource(r, **v))
-
+        resources_opts = self._map_resources_to_jobopts()
         self._job.options = resources_opts + self._job.options
         with osext.change_dir(self._stagedir):
             try:
@@ -1949,6 +1947,13 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
         if self.job.sched_flex_alloc_nodes:
             self.num_tasks = self.job.num_tasks
 
+    def _map_resources_to_jobopts(self):
+        resources_opts = []
+        for r, v in self.extra_resources.items():
+            resources_opts += self._current_partition.get_resource(r, **v)
+
+        return resources_opts
+
     @final
     def compile_complete(self):
         '''Check if the build phase has completed.
@@ -1995,17 +2000,6 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
 
         return self._job.finished()
 
-    @final
-    def poll(self):
-        '''See :func:`run_complete`.
-
-        .. deprecated:: 3.2
-
-        '''
-        user_deprecation_warning('calling poll() is deprecated; '
-                                 'please use run_complete() instead')
-        return self.run_complete()
-
     @final
     def run_wait(self):
         '''Wait for the run phase of this test to finish.
@@ -2026,16 +2020,6 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
         '''
         self._job.wait()
 
-    @final
-    def wait(self):
-        '''See :func:`run_wait`.
-
-        .. deprecated:: 3.2
-        '''
-        user_deprecation_warning('calling wait() is deprecated; '
-                                 'please use run_wait() instead')
-        self.run_wait()
-
     @final
     def sanity(self):
         self.check_sanity()
@@ -2100,6 +2084,10 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
             if not success:
                 raise SanityError()
 
+    def is_performance_check(self):
+        '''Return :obj:`True` if the test is a performance test.'''
+        return self.perf_variables or hasattr(self, 'perf_patterns')
+
     @final
     def check_performance(self):
         '''The performance checking phase of the regression test pipeline.
@@ -2122,98 +2110,74 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
 
         '''
 
+        if not self.is_performance_check():
+            return
+
+        perf_patterns = getattr(self, 'perf_patterns', None)
+
+        if perf_patterns is not None and self.perf_variables:
+            raise ReframeSyntaxError(
+                f"you cannot mix 'perf_patterns' and 'perf_variables' syntax"
+            )
+
+        # Convert `perf_patterns` to `perf_variables`
+        if perf_patterns:
+            for var, expr in self.perf_patterns.items():
+                # Retrieve the unit from the reference tuple
+                key = f'{self._current_partition.fullname}:{var}'
+                try:
+                    unit = self.reference[key][3]
+                    if unit is None:
+                        unit = ''
+                except KeyError:
+                    unit = ''
+
+                self.perf_variables[var] = sn.make_performance_function(expr,
+                                                                        unit)
+
+        # Evaluate the performance function and retrieve the metrics
         with osext.change_dir(self._stagedir):
-            if self.perf_variables or self._rfm_perf_fns:
-                if hasattr(self, 'perf_patterns'):
-                    raise ReframeSyntaxError(
-                        f"assigning a value to 'perf_patterns' conflicts ",
-                        f"with using the 'performance_function' decorator ",
-                        f"or setting a value to 'perf_variables'"
+            for tag, expr in self.perf_variables.items():
+                try:
+                    value = expr.evaluate()
+                    unit = expr.unit
+                except Exception as e:
+                    logging.getlogger().warning(
+                        f'skipping evaluation of performance variable '
+                        f'{tag!r}: {e}'
                     )
+                    continue
 
-                # Log the performance variables
-                self._setup_perf_logging()
-                for tag, expr in self.perf_variables.items():
-                    try:
-                        value = expr.evaluate()
-                        unit = expr.unit
-                    except Exception as e:
-                        logging.getlogger().warning(
-                            f'skipping evaluation of performance variable '
-                            f'{tag!r}: {e}'
-                        )
-                        continue
+                key = f'{self._current_partition.fullname}:{tag}'
+                try:
+                    ref = self.reference[key]
 
-                    key = f'{self._current_partition.fullname}:{tag}'
-                    try:
-                        ref = self.reference[key]
+                    # If units are also provided in the reference, raise
+                    # a warning if they match with the units provided by
+                    # the performance function.
+                    if len(ref) == 4:
+                        if ref[3] != unit:
+                            logging.getlogger().warning(
+                                f'reference unit ({key!r}) for the '
+                                f'performance variable {tag!r} '
+                                f'does not match the unit specified '
+                                f'in the performance function ({unit!r}): '
+                                f'{unit!r} will be used'
+                            )
 
-                        # If units are also provided in the reference, raise
-                        # a warning if they match with the units provided by
-                        # the performance function.
-                        if len(ref) == 4:
-                            if ref[3] != unit:
-                                logging.getlogger().warning(
-                                    f'reference unit ({key!r}) for the '
-                                    f'performance variable {tag!r} '
-                                    f'does not match the unit specified '
-                                    f'in the performance function ({unit!r}): '
-                                    f'{unit!r} will be used'
-                                )
+                        # Pop the unit from the ref tuple (redundant)
+                        ref = ref[:3]
+                except KeyError:
+                    if self.require_reference:
+                        raise PerformanceError(
+                            f'no reference value found for '
+                            f'performance variable {tag!r} on '
+                            f'system {self._current_partition.fullname!r}'
+                        ) from None
 
-                            # Pop the unit from the ref tuple (redundant)
-                            ref = ref[:3]
-                    except KeyError:
-                        ref = (0, None, None)
+                    ref = (0, None, None)
 
-                    self._perfvalues[key] = (value, *ref, unit)
-                    self._perf_logger.log_performance(logging.INFO, tag, value,
-                                                      *ref, unit)
-            elif not hasattr(self, 'perf_patterns'):
-                return
-            else:
-                self._setup_perf_logging()
-                # Check if default reference perf values are provided and
-                # store all the variables tested in the performance check
-                has_default = False
-                variables = set()
-                for key, ref in self.reference.items():
-                    keyparts = key.split(self.reference.scope_separator)
-                    system = keyparts[0]
-                    varname = keyparts[-1]
-                    unit = ref[3]
-                    variables.add((varname, unit))
-                    if system == '*':
-                        has_default = True
-                        break
-
-                if not has_default:
-                    if not variables:
-                        # If empty, it means that self.reference was empty, so
-                        # try to infer their name from perf_patterns
-                        variables = {(name, None)
-                                     for name in self.perf_patterns.keys()}
-
-                    for var in variables:
-                        name, unit = var
-                        ref_tuple = (0, None, None, unit)
-                        self.reference.update({'*': {name: ref_tuple}})
-
-                # We first evaluate and log all performance values and then we
-                # check them against the reference. This way we always log them
-                # even if the don't meet the reference.
-                for tag, expr in self.perf_patterns.items():
-                    value = sn.evaluate(expr)
-                    key = f'{self._current_partition.fullname}:{tag}'
-                    if key not in self.reference:
-                        raise SanityError(
-                            f'tag {tag!r} not resolved in references for '
-                            f'{self._current_partition.fullname}'
-                        )
-
-                    self._perfvalues[key] = (value, *self.reference[key])
-                    self._perf_logger.log_performance(logging.INFO, tag, value,
-                                                      *self.reference[key])
+                self._perfvalues[key] = (value, *ref, unit)
 
             # Check the performance variables against their references.
             for key, values in self._perfvalues.items():
@@ -2235,7 +2199,7 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
                                  'expected {1} (l={2}, u={3})' % tag))
                     )
                 except SanityError as e:
-                    raise PerformanceError(e)
+                    raise PerformanceError(e) from None
 
     def _copy_job_files(self, job, dst):
         if job is None:
@@ -2309,42 +2273,6 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
     def user_deps(self):
         return util.SequenceView(self._userdeps)
 
-    def _depends_on_func(self, how, subdeps=None, *args, **kwargs):
-        if args or kwargs:
-            raise ValueError('invalid arguments passed')
-
-        user_deprecation_warning("passing 'how' as an integer or passing "
-                                 "'subdeps' is deprecated; please have a "
-                                 "look at the user documentation")
-
-        if (subdeps is not None and
-            not isinstance(subdeps, typ.Dict[str, typ.List[str]])):
-            raise TypeError("subdeps argument must be of type "
-                            "`Dict[str, List[str]]' or `None'")
-
-        # Now return a proper when function
-        def exact(src, dst):
-            if not subdeps:
-                return False
-
-            p0, e0 = src
-            p1, e1 = dst
-
-            # DEPEND_EXACT allows dependencies inside the same partition
-            return ((p0 == p1) and (e0 in subdeps) and (e1 in subdeps[e0]))
-
-        # Follow the old definitions
-        # DEPEND_BY_ENV used to mean same env and same partition
-        if how == DEPEND_BY_ENV:
-            return udeps.by_case
-        # DEPEND_BY_ENV used to mean same partition
-        elif how == DEPEND_FULLY:
-            return udeps.by_part
-        elif how == DEPEND_EXACT:
-            return exact
-        else:
-            raise ValueError(f"unknown value passed to 'how' argument: {how}")
-
     def depends_on(self, target, how=None, *args, **kwargs):
         '''Add a dependency to another test.
 
@@ -2404,15 +2332,13 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
             Passing an integer to the ``how`` argument as well as using the
             ``subdeps`` argument is deprecated.
 
+        .. versionchanged:: 4.0.0
+           Passing an integer to the ``how`` argument is no longer supported.
+
         '''
         if not isinstance(target, str):
             raise TypeError("target argument must be of type: `str'")
 
-        if (isinstance(how, int)):
-            # We are probably using the old syntax; try to get a
-            # proper how function
-            how = self._depends_on_func(how, *args, **kwargs)
-
         if how is None:
             how = udeps.by_case
 
