@@ -7,6 +7,7 @@ import inspect
 import itertools
 import json
 import os
+import random
 import shlex
 import socket
 import sys
@@ -18,13 +19,13 @@ import reframe.core.config as config
 import reframe.core.exceptions as errors
 import reframe.core.logging as logging
 import reframe.core.runtime as runtime
-import reframe.core.warnings as warnings
 import reframe.frontend.argparse as argparse
 import reframe.frontend.autodetect as autodetect
 import reframe.frontend.ci as ci
 import reframe.frontend.dependencies as dependencies
 import reframe.frontend.filters as filters
 import reframe.frontend.runreport as runreport
+import reframe.utility as util
 import reframe.utility.jsonext as jsonext
 import reframe.utility.osext as osext
 import reframe.utility.typecheck as typ
@@ -71,6 +72,7 @@ def list_checks(testcases, printer, detailed=False, concretized=False):
                 unique_checks.add(v.check.unique_name)
 
         if depth:
+            name_info = f'{u.check.display_name} /{u.check.hashcode}'
             tc_info = ''
             details = ''
             if concretized:
@@ -78,17 +80,16 @@ def list_checks(testcases, printer, detailed=False, concretized=False):
 
             location = inspect.getfile(type(u.check))
             if detailed:
-                details = f' [id: {u.check.unique_name}, file: {location!r}]'
+                details = f' [variant: {u.check.variant_num}, file: {location!r}]'
 
-            lines.append(
-                f'{prefix}^{u.check.display_name}{tc_info}{details}'
-            )
+            lines.append(f'{prefix}^{name_info}{tc_info}{details}')
 
         return lines
 
     # We need the leaf test cases to be printed at the leftmost
     leaf_testcases = list(t for t in testcases if t.in_degree == 0)
     for t in leaf_testcases:
+        name_info = f'{t.check.display_name} /{t.check.hashcode}'
         tc_info = ''
         details = ''
         if concretized:
@@ -96,12 +97,11 @@ def list_checks(testcases, printer, detailed=False, concretized=False):
 
         location = inspect.getfile(type(t.check))
         if detailed:
-            details = f' [id: {t.check.unique_name}, file: {location!r}]'
+            details = f' [variant: {t.check.variant_num}, file: {location!r}]'
 
-        # if not concretized and t.check.name not in unique_checks:
         if concretized or (not concretized and
                            t.check.unique_name not in unique_checks):
-            printer.info(f'- {t.check.display_name}{tc_info}{details}')
+            printer.info(f'- {name_info}{tc_info}{details}')
 
         if not t.check.is_fixture():
             unique_checks.add(t.check.unique_name)
@@ -154,8 +154,9 @@ def describe_checks(testcases, printer):
 
             # List all required variables
             required = []
-            for var in tc.check._rfm_var_space:
-                if not tc.check._rfm_var_space[var].is_defined():
+            var_space = type(tc.check).var_space
+            for var in var_space:
+                if not var_space[var].is_defined():
                     required.append(var)
 
             rec['@required'] = required
@@ -281,13 +282,6 @@ def main():
         help="Add PATH to the check search path list",
         envvar='RFM_CHECK_SEARCH_PATH :', configvar='general/check_search_path'
     )
-    locate_options.add_argument(
-        '--ignore-check-conflicts', action='store_true',
-        help=('Skip checks with conflicting names '
-              '(this option is deprecated and has no effect)'),
-        envvar='RFM_IGNORE_CHECK_CONFLICTS',
-        configvar='general/ignore_check_conflicts'
-    )
     locate_options.add_argument(
         '-R', '--recursive', action='store_true',
         help='Search for checks in the search path recursively',
@@ -386,6 +380,11 @@ def main():
         help=('Distribute the selected single-node jobs on every node that'
               'is in STATE (default: "idle"')
     )
+    run_options.add_argument(
+        '--exec-order', metavar='ORDER', action='store',
+        choices=['name', 'random', 'rname', 'ruid', 'uid'],
+        help='Impose an execution order for independent tests'
+    )
     run_options.add_argument(
         '--exec-policy', metavar='POLICY', action='store',
         choices=['async', 'serial'], default='async',
@@ -396,10 +395,6 @@ def main():
         dest='flex_alloc_nodes', metavar='{all|STATE|NUM}', default=None,
         help='Set strategy for the flexible node allocation (default: "idle").'
     )
-    run_options.add_argument(
-        '--force-local', action='store_true',
-        help='Force local execution of checks'
-    )
     run_options.add_argument(
         '-J', '--job-option', action='append', metavar='OPT',
         dest='job_options', default=[],
@@ -448,10 +443,6 @@ def main():
         '--skip-system-check', action='store_true',
         help='Skip system check'
     )
-    run_options.add_argument(
-        '--strict', action='store_true',
-        help='Enforce strict performance checking'
-    )
 
     # Environment options
     env_options.add_argument(
@@ -496,10 +487,10 @@ def main():
 
     # Miscellaneous options
     misc_options.add_argument(
-        '-C', '--config-file', action='store',
-        dest='config_file', metavar='FILE',
+        '-C', '--config-file', action='append', metavar='FILE',
+        dest='config_files',
         help='Set configuration file',
-        envvar='RFM_CONFIG_FILE'
+        envvar='RFM_CONFIG_FILES :'
     )
     misc_options.add_argument(
         '--detect-host-topology', action='store', nargs='?', const='-',
@@ -526,10 +517,6 @@ def main():
         '--system', action='store', help='Load configuration for SYSTEM',
         envvar='RFM_SYSTEM'
     )
-    misc_options.add_argument(
-        '--upgrade-config-file', action='store', metavar='OLD[:NEW]',
-        help='Upgrade ReFrame 2.x configuration file to ReFrame 3.x syntax'
-    )
     misc_options.add_argument(
         '-V', '--version', action='version', version=osext.reframe_version()
     )
@@ -548,9 +535,9 @@ def main():
         dest='autodetect_fqdn',
         envvar='RFM_AUTODETECT_FQDN',
         action='store',
-        default=True,
+        default=False,
         type=typ.Bool,
-        help='Use FQDN as host name'
+        help='Use the full qualified domain name as host name'
     )
     argparser.add_argument(
         dest='autodetect_method',
@@ -559,13 +546,19 @@ def main():
         default='hostname',
         help='Method to detect the system'
     )
+    argparser.add_argument(
+        dest='config_path',
+        envvar='RFM_CONFIG_PATH :',
+        action='append',
+        help='Directories where ReFrame will look for base configuration'
+    )
     argparser.add_argument(
         dest='autodetect_xthostname',
         envvar='RFM_AUTODETECT_XTHOSTNAME',
         action='store',
-        default=True,
+        default=False,
         type=typ.Bool,
-        help="Use Cray's xthostname file to find the host name"
+        help="Use Cray's xthostname file to retrieve the host name"
     )
     argparser.add_argument(
         dest='git_timeout',
@@ -576,12 +569,6 @@ def main():
               'valid repository.'),
         type=float
     )
-    argparser.add_argument(
-        dest='graylog_server',
-        envvar='RFM_GRAYLOG_ADDRESS',
-        configvar='logging/handlers_perflog/graylog_address',
-        help='Graylog server address'
-    )
     argparser.add_argument(
         dest='httpjson_url',
         envvar='RFM_HTTPJSON_URL',
@@ -591,16 +578,9 @@ def main():
     argparser.add_argument(
         dest='ignore_reqnodenotavail',
         envvar='RFM_IGNORE_REQNODENOTAVAIL',
-        configvar='schedulers/ignore_reqnodenotavail',
+        configvar='systems*/sched_options/ignore_reqnodenotavail',
         action='store_true',
-        help='Graylog server address'
-    )
-    argparser.add_argument(
-        dest='compact_test_names',
-        envvar='RFM_COMPACT_TEST_NAMES',
-        configvar='general/compact_test_names',
-        action='store_true',
-        help='Use a compact test naming scheme'
+        help='Ignore ReqNodeNotAvail Slurm error'
     )
     argparser.add_argument(
         dest='dump_pipeline_progress',
@@ -609,6 +589,15 @@ def main():
         action='store_true',
         help='Dump progress information for the async execution'
     )
+    argparser.add_argument(
+        dest='perf_info_level',
+        envvar='RFM_PERF_INFO_LEVEL',
+        configvar='general/perf_info_level',
+        action='store',
+        type=typ.Str[r'critical|error|warning|info|verbose|'
+                     r'debug|debug2|undefined'],
+        help='Log level at which immediate performance info will be printed'
+    )
     argparser.add_argument(
         dest='pipeline_timeout',
         envvar='RFM_PIPELINE_TIMEOUT',
@@ -698,45 +687,26 @@ def main():
     if not restrict_logging():
         printer.adjust_verbosity(calc_verbosity(site_config, options.quiet))
 
-    if os.getenv('RFM_GRAYLOG_SERVER'):
-        printer.warning(
-            'RFM_GRAYLOG_SERVER environment variable is deprecated; '
-            'please use RFM_GRAYLOG_ADDRESS instead'
-        )
-        os.environ['RFM_GRAYLOG_ADDRESS'] = os.getenv('RFM_GRAYLOG_SERVER')
-
-    if options.upgrade_config_file is not None:
-        old_config, *new_config = options.upgrade_config_file.split(
-            ':', maxsplit=1
-        )
-        new_config = new_config[0] if new_config else None
-
-        try:
-            new_config = config.convert_old_config(old_config, new_config)
-        except Exception as e:
-            printer.error(f'could not convert file: {e}')
-            sys.exit(1)
-
-        printer.info(
-            f'Conversion successful! '
-            f'The converted file can be found at {new_config!r}.'
-        )
-        sys.exit(0)
-
     # Now configure ReFrame according to the user configuration file
     try:
-        try:
-            printer.debug('Loading user configuration')
-            site_config = config.load_config(options.config_file)
-        except warnings.ReframeDeprecationWarning as e:
-            printer.warning(e)
-            converted = config.convert_old_config(options.config_file)
-            printer.warning(
-                f"configuration file has been converted "
-                f"to the new syntax here: '{converted}'"
-            )
-            site_config = config.load_config(converted)
+        # Issue a deprecation warning if the old `RFM_CONFIG_FILE` is used
+        config_file = os.getenv('RFM_CONFIG_FILE')
+        if config_file is not None:
+            printer.warning('RFM_CONFIG_FILE is deprecated; '
+                            'please use RFM_CONFIG_FILES instead')
+            if os.getenv('RFM_CONFIG_FILES'):
+                printer.warning(
+                    'both RFM_CONFIG_FILE and RFM_CONFIG_FILES are specified; '
+                    'the former will be ignored'
+                )
+            else:
+                os.environ['RFM_CONFIG_FILES'] = config_file
 
+        printer.debug('Loading user configuration')
+        conf_files = config.find_config_files(
+            options.config_path, options.config_files
+        )
+        site_config = config.load_config(*conf_files)
         site_config.validate()
         site_config.set_autodetect_meth(
             options.autodetect_method,
@@ -769,7 +739,7 @@ def main():
         logging.configure_logging(site_config)
     except (OSError, errors.ConfigError) as e:
         printer.error(f'failed to load configuration: {e}')
-        printer.error(logfiles_message())
+        printer.info(logfiles_message())
         sys.exit(1)
 
     printer.colorize = site_config.get('general/0/colorize')
@@ -781,15 +751,9 @@ def main():
         runtime.init_runtime(site_config)
     except errors.ConfigError as e:
         printer.error(f'failed to initialize runtime: {e}')
-        printer.error(logfiles_message())
+        printer.info(logfiles_message())
         sys.exit(1)
 
-    if site_config.get('general/0/ignore_check_conflicts'):
-        logging.getlogger().warning(
-            "the 'ignore_check_conflicts' option is deprecated "
-            "and will be removed in the future"
-        )
-
     rt = runtime.runtime()
     try:
         if site_config.get('general/0/module_map_file'):
@@ -810,7 +774,7 @@ def main():
         printer.error("stage and output refer to the same directory; "
                       "if this is on purpose, please use the "
                       "'--keep-stage-files' option.")
-        printer.error(logfiles_message())
+        printer.info(logfiles_message())
         sys.exit(1)
 
     # Show configuration after everything is set up
@@ -926,7 +890,7 @@ def main():
 
     session_info = {
         'cmdline': ' '.join(sys.argv),
-        'config_file': rt.site_config.filename,
+        'config_files': rt.site_config.sources,
         'data_version': runreport.DATA_VERSION,
         'hostname': socket.gethostname(),
         'prefix_output': rt.output_prefix,
@@ -945,12 +909,17 @@ def main():
         f"{session_info['user'] or '<unknown>'}@{session_info['hostname']}"
     )
     print_infoline('working directory', repr(session_info['workdir']))
-    print_infoline('settings file', f"{session_info['config_file']!r}")
+    print_infoline(
+        'settings files',
+        ', '.join(repr(x) for x in session_info['config_files'])
+    )
     print_infoline('check search path',
                    f"{'(R) ' if loader.recurse else ''}"
                    f"{':'.join(loader.load_path)!r}")
     print_infoline('stage directory', repr(session_info['prefix_stage']))
     print_infoline('output directory', repr(session_info['prefix_output']))
+    print_infoline('log files',
+                   ', '.join(repr(s) for s in logging.log_files()))
     printer.info('')
     try:
         logging.getprofiler().enter_region('test processing')
@@ -981,6 +950,9 @@ def main():
         testcases = testcases_all
         printer.verbose(f'Generated {len(testcases)} test case(s)')
 
+        # Filter out fixtures
+        testcases = [t for t in testcases if not t.check.is_fixture()]
+
         # Filter test cases by name
         if options.exclude_names:
             for name in options.exclude_names:
@@ -1061,8 +1033,8 @@ def main():
                     "a non-negative integer"
                 ) from None
 
-            testcases = repeat_tests(testcases, num_repeats)
-            testcases_all = testcases
+            testcases_all = repeat_tests(testcases, num_repeats)
+            testcases = testcases_all
 
         if options.distribute:
             node_map = getallnodes(options.distribute, parsed_job_options)
@@ -1077,15 +1049,31 @@ def main():
                 x for x in parsed_job_options
                 if (not x.startswith('-w') and not x.startswith('--nodelist'))
             ]
-            testcases = distribute_tests(testcases, node_map)
-            testcases_all = testcases
+            testcases_all = distribute_tests(testcases, node_map)
+            testcases = testcases_all
+
+        @logging.time_function
+        def _sort_testcases(testcases):
+            if options.exec_order in ('name', 'rname'):
+                testcases.sort(key=lambda c: c.check.display_name,
+                               reverse=(options.exec_order == 'rname'))
+            elif options.exec_order in ('uid', 'ruid'):
+                testcases.sort(key=lambda c: c.check.unique_name,
+                               reverse=(options.exec_order == 'ruid'))
+            elif options.exec_order == 'random':
+                random.shuffle(testcases)
+
+        _sort_testcases(testcases)
+        if testcases_all is not testcases:
+            _sort_testcases(testcases_all)
 
         # Prepare for running
         printer.debug('Building and validating the full test DAG')
         testgraph, skipped_cases = dependencies.build_deps(testcases_all)
         if skipped_cases:
             # Some cases were skipped, so adjust testcases
-            testcases = list(set(testcases) - set(skipped_cases))
+            testcases = list(util.OrderedSet(testcases) -
+                             util.OrderedSet(skipped_cases))
             printer.verbose(
                 f'Filtering test case(s) due to unresolved dependencies: '
                 f'{len(testcases)} remaining'
@@ -1224,11 +1212,12 @@ def main():
             try:
                 rt.modules_system.load_module(**m, force=True)
             except errors.EnvironError as e:
-                printer.warning(
-                    f'could not load module {m["name"]!r} correctly; '
-                    f'skipping...'
+                printer.error(
+                    f'could not load module {m["name"]!r} correctly; rerun '
+                    f'with -vv for more information'
                 )
                 printer.debug(str(e))
+                sys.exit(1)
 
         options.flex_alloc_nodes = options.flex_alloc_nodes or 'idle'
 
@@ -1245,8 +1234,6 @@ def main():
             printer.error("unknown execution policy `%s': Exiting...")
             sys.exit(1)
 
-        exec_policy.force_local = options.force_local
-        exec_policy.strict_check = options.strict
         exec_policy.skip_sanity_check = options.skip_sanity_check
         exec_policy.skip_performance_check = options.skip_performance_check
         exec_policy.keep_stage_files = site_config.get(
@@ -1378,7 +1365,6 @@ def main():
             log_files = logging.log_files()
             if site_config.get('general/0/save_log_files'):
                 log_files = logging.save_log_files(rt.output_prefix)
-
         except OSError as e:
             printer.error(f'could not save log file: {e}')
             sys.exit(1)
