@@ -156,6 +156,12 @@ class RegressionTask:
 
         self._aborted = False
 
+        # Performance logging
+        self._perflogger = logging.null_logger
+        self._perflog_compat = runtime.runtime().get_option(
+            'logging/0/perflog_compat'
+        )
+
     def duration(self, phase):
         # Treat pseudo-phases first
         if phase == 'compile_complete':
@@ -362,6 +368,9 @@ class RegressionTask:
 
     @logging.time_function
     def performance(self):
+        if self.check.is_performance_check():
+            self._perflogger = logging.getperflogger(self.check)
+
         self._safe_call(self.check.performance)
 
     @logging.time_function
@@ -377,6 +386,9 @@ class RegressionTask:
 
         self._current_stage = 'finalize'
         self._notify_listeners('on_task_success')
+        if self.check.is_performance_check():
+            self._perflogger.log_performance(logging.INFO, self,
+                                             multiline=self._perflog_compat)
 
     @logging.time_function
     def cleanup(self, *args, **kwargs):
@@ -386,6 +398,9 @@ class RegressionTask:
         self._failed_stage = self._current_stage
         self._exc_info = exc_info or sys.exc_info()
         self._notify_listeners('on_task_failure')
+        if self.check.is_performance_check():
+            self._perflogger.log_performance(logging.INFO, self,
+                                             multiline=self._perflog_compat)
 
     def skip(self, exc_info=None):
         self._skipped = True
@@ -416,9 +431,10 @@ class RegressionTask:
     def info(self):
         '''Return an info string about this task.'''
         name = self.check.display_name
+        hashcode = self.check.hashcode
         part = self.testcase.partition.fullname
         env  = self.testcase.environ.name
-        return f'{name} @{part}+{env}'
+        return f'{name} /{hashcode} @{part}+{env}'
 
 
 class TaskEventListener(abc.ABC):
@@ -572,13 +588,11 @@ class ExecutionPolicy(abc.ABC):
 
     def __init__(self):
         # Options controlling the check execution
-        self.force_local = False
         self.skip_sanity_check = False
         self.skip_performance_check = False
         self.keep_stage_files = False
         self.only_environs = None
         self.printer = None
-        self.strict_check = False
 
         # Local scheduler for running forced local jobs
         self.local_scheduler = LocalJobScheduler()
@@ -600,9 +614,3 @@ class ExecutionPolicy(abc.ABC):
     @abc.abstractmethod
     def runcase(self, case):
         '''Run a test case.'''
-
-        if self.strict_check:
-            case.check.strict_check = True
-
-        if self.force_local:
-            case.check.local = True
